<?php
/**
 * Nic_Storage_SQL:: SQL implementing storage backends for Network.
 *
 * $Horde: nic/lib/Storage/sql.php,v 1.8 2004/10/31 10:32:55 jan Exp $
 *
 * @author  Mike Cochrane <mike@graftonhall.co.nz>
 * @version $Revision: 1.8 $
 * @since   Network 0.1
 * @package Nic
 */
class Nic_Storage_SQL extends Nic_Storage {

    /** Hash containing connection parameters. */
    var $_params = array();

    /** Handle for the current database connection.
        @var object DB $db */
    var $_db;

    /** Boolean indicating whether or not we're connected to the SQL server. */
    var $_connected = false;

    /**
     * Constructs a new SQL storage object.
     *
     * @param string $device    Device ID to store / retrive values for.
     * @param array  $params    A hash containing connection parameters.
     */
    function Nic_Storage_SQL($params)
    {
        $this->_params = $params;
    }

    /**
     * Calculate the value of a counter at a time
     *
     * @param int       $time       The time to get a value for
     * @param array     $values     Array of values KEY: time VALUE: value
     *
     * @return int      Value at this time
     */
    function _interpolate($time, $values)
    {
        if (array_key_exists($time, $values)) {
            return $values[$time];
        } else {
            $before_key = min(array_keys($values));
            $after_key = max(array_keys($values));

            ksort ($values);
            reset ($values);
            while (list ($key, $val) = each ($values)) {
                if ($key > $before_key && $key < $time) {
                    $before_key = $key;
                } elseif ($key < $after_key && $key > $time) {
                    $after_key = $key;
                }
            }

            $period = $after_key - $before_key;
            if ($period == 0) {
                $value = $values[$before_key];
            } else {
                $value = $values[$before_key] + ($values[$after_key] - $values[$before_key]) * (($time - $before_key) / ($period));
            }
            return max(0, $value);
        }
    }

    /**
     * Retrieves data values for the specified time interval as given steps
     * Where exact values aren't available they are linearly interpolated.
     *
     * @param string    $name       The name of the counter to retrieve.
     * @param constant  $dataType   The type of data being retrieved.
     * @param int       $startTime  The start time of the range to retrieve.
     * @param int       $endTime    The end time of the range to retieve
     *                              default is time();
     * @param int       $step       (optional) The step (in seconds) between values
     *                              to be retrieved. Default is 300 (5 mins);
     *
     * @return array    Array KEY: Time VALUE: Data Value.
     */
    function fetch($hostname, $subsystem, $item, $dataType, $startTime, $endTime, $step = 300)
    {
        /* Make sure we have a valid database connection. */
        $this->_connect();

        /* Build the SQL query. */
        $query = sprintf('SELECT * FROM %s WHERE data_hostname = %s AND data_subsystem = %s AND data_item = %s AND data_time >= %d AND data_time <= %d',
                         $this->_params['table'],
                         $this->_db->quote($hostname),
                         $this->_db->quote($subsystem),
                         $this->_db->quote($item),
                         $startTime - 3600,
                         $endTime + 3600);

        /* Execute the query. */
        $result = $this->_db->query($query);
        $values = array();

        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
        if (is_a($row, 'PEAR_Error')) {
            return $row;
        }

        while ($row && !is_a($row, 'PEAR_Error') && is_array($row)) {
            $values[$row['data_time']] = $row['data_value'];

            if (strstr($values[$row['data_time']], 'Counter32:') !== false) {
                $values[$row['data_time']] = trim(substr($values[$row['data_time']], 10));
            }


            /* Advance to the new row in the result set. */
            $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
        }

        /* Transform a counter into a rate for display. */
        if ($dataType == NIC_DATA_COUNTER) {
            $temp = array();

            ksort($values);
            $prev_key = min(array_keys($values));
            next ($values);

            while (list ($key, $val) = each ($values)) {
                $temp[($key + $prev_key) / 2] = ($values[$key] - $values[$prev_key]) / ($key - $prev_key);
                $prev_key = $key;
            }

            $values = $temp;
        }

        /** fill in the gaps */
        $temp = array();
        if (count($values) > 0) {
            for ($i = $startTime; $i <= $endTime; $i += $step) {
                if ($dataType == NIC_DATA_INTEGER) {
                    $temp[$i] = abs($this->_interpolate($i, $values));
                } else {
                    $temp[$i] = $this->_interpolate($i, $values);
                }
            }
        }
        $values = $temp;

        return $values;
    }

    /**
     * Add a new data value to the storage system.
     *
     * @param string    $name       The name of the counter to retrieve.
     * @param counstant $dataType   The type of data being retrieved.
     * @param int       $time       The time the reading was taken.
     * @param int       $value      The value to add.
     *
     * @return mixed    (boolean) true on success, PEAR_Error on failure.
     */
    function add($time, $hostname, $subsystem, $item, $value)
    {
        /* Make sure we have a valid database connection. */
        $this->_connect();

        $query = 'INSERT INTO ' . $this->_params['table'] . '(data_time, data_hostname, data_subsystem, data_item, data_value)';
        $query .= sprintf("VALUES (%d, %s, %s, %s, %s)", $time, $this->_db->quote($hostname), $this->_db->quote($subsystem), $this->_db->quote($item), $this->_db->quote($value));

        /* Attempt the insertion query. */
        $result = $this->_db->query($query);

        /* Return an error immediately if the query failed. */
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        return true;
    }

    /**
     * Remove old status records from database.
     *
     * @param int        $removebefore  Remove older records 
     *
     * @return mixed     (boolean) true on success, PEAR_Error on failure.
     */
    function purge($removebefore)
    {
        /* Make sure we have a valid database connection. */
        $this->_connect();
                                                                                                                                                             
        $query = "DELETE FROM " . $this->_params['table'] . ' WHERE data_time < ' . $removebefore; 

        $result = $this->_db->query($query);

        /* Return an error immediately if the query failed. */
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }
                                                                                                                                                             
        return true;
    }

    /**
     * Return the oldest record from the database
     *
     * @return longint  Oldest timestamp in the table
     */
    function oldest()
    {
        /* Make sure we have a valid database connection. */
        $this->_connect();
                                                                                                                                                             
        $query = "SELECT MIN(data_time) as oldest from " . $this->_params['table'];
                                                                                                                                                             
        $result = $this->_db->query($query);
                                                                                                                                                             
        /* Return an error immediately if the query failed. */
        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        /* Advance to the new row in the result set. */
        $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
     
        return $row['oldest'];
    }



    /**
     * Attempts to open a persistent connection to the SQL server.
     *
     * @return boolean    True on success; exits (Horde::fatal()) on error.
     */
    function _connect()
    {
        if (!$this->_connected) {
            Horde::assertDriverConfig($this->_params, 'storage',
                array('phptype', 'hostspec', 'username', 'database', 'table'),
                'nic storage SQL');

            /* Connect to the SQL server using the supplied parameters. */
            require_once 'DB.php';
            $this->_db = &DB::connect($this->_params,
                                     array('persistent' => !empty($this->_params['persistent'])));
            if (is_a($this->_db, 'PEAR_Error')) {
                Horde::fatal($this->_db, __FILE__, __LINE__);
            }

            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);

            $this->_connected = true;
        }

        return true;
    }

}
